@@ -150,6 +150,7 @@ const char *script::checks::chkApply(script *img, script::header *scr, script::l
 	if(!tmp)
 		return "invalid or unknown script applied";
 
+	scr->methods = tmp->methods;
 	scr->events = tmp->events;
 	return NULL;
 }
@@ -237,6 +238,18 @@ const char *script::checks::chkBreak(script *img, script::header *scr, script::l
 	return NULL;
 }
 
+const char *script::checks::chkRef(script *img, script::header *scr, script::line_t *line)
+{
+	if(line->argc != 1)
+		return "one symbol argument for referencing";
+
+	const char *cp = line->argv[0];
+	if(*cp != '$')
+		return "only field operations can be referenced alone";
+
+	return NULL;
+}
+
 const char *script::checks::chkWhen(script *img, script::header *scr, script::line_t *line)
 {
 	if(!line->argc)
@@ -250,6 +263,9 @@ const char *script::checks::chkIf(script *img, script::header *scr, script::line
 	if(!img->push(line))
 		return "analysis overflow for if command";
 
+	if(img->thencheck)
+		return "cannot nest if in if-then clause";
+
 	return chkConditional(img, scr, line);
 }
 
@@ -258,6 +274,9 @@ const char *script::checks::chkWhile(script *img, script::header *scr, script::l
 	if(String::equal(scr->name, "_init_"))
 		return "this command cannot be used to initialize";
 
+	if(img->thencheck)
+		return "cannot use while in if-then clause";
+
 	if(!img->push(line))
 		return "stack overflow for while command";
 
@@ -286,7 +305,7 @@ const char *script::checks::chkExpand(script *img, script::header *scr, script::
 		case '=':
 			return "no keywords are used in this command";
 		case '$':
-			return "cannot assign fielded symbols";
+			return "cannot assign to format";
 		case '@':
 			return "cannot assign to label";
 		case '^':
@@ -306,6 +325,9 @@ const char *script::checks::chkForeach(script *img, script::header *scr, script:
 	if(String::equal(scr->name, "_init_"))
 		return "this command cannot be used to initialize";
 
+	if(img->thencheck)
+		return "cannot use for in if-then clause";
+
 	if(!img->push(line))
 		return "stack overflow for do command";
 
@@ -322,7 +344,7 @@ const char *script::checks::chkForeach(script *img, script::header *scr, script:
 	case '=':
 		return "no keywords are used in this command";
 	case '$':
-		return "cannot assign fielded symbols";
+		return "cannot assign to format";
 	case '@':
 		return "cannot assign to label";
 	case '^':
@@ -346,6 +368,9 @@ const char *script::checks::chkDo(script *img, script::header *scr, script::line
 	if(String::equal(scr->name, "_init_"))
 		return "this command cannot be used to initialize";
 
+	if(img->thencheck)
+		return "cannot use do in if-then clause";
+
 	if(line->argc)
 		return "no arguments for do command";
 
@@ -359,6 +384,9 @@ const char *script::checks::chkEndif(script *img, script::header *scr, script::l
 {
 	script::method_t method = img->pull();
 
+	if(img->thencheck)
+		return "cannot endif in if-then clause";
+
 	if(method != (method_t)&script::methods::scrIf && method != (method_t)&script::methods::scrElse)
 		return "endif not within an if block";
 
@@ -372,6 +400,9 @@ const char *script::checks::chkEndcase(script *img, script::header *scr, script:
 {
 	script::method_t method = img->pull();
 
+	if(img->thencheck)
+		return "cannot endcase in if-then clause";
+
 	if(String::equal(scr->name, "_init_"))
 		return "this command cannot be used to initialize";
 
@@ -388,6 +419,9 @@ const char *script::checks::chkEndcase(script *img, script::header *scr, script:
 const char *script::checks::chkLoop(script *img, script::header *scr, script::line_t *line)
 {
 	script::method_t method = img->pull();
+	
+	if(img->thencheck)
+		return "can not end loop in if-then clause";
 
 	if(String::equal(scr->name, "_init_"))
 		return "this command cannot be used to initialize";
@@ -417,6 +451,9 @@ const char *script::checks::chkCase(script *img, script::header *scr, script::li
 {
 	script::method_t method = img->looping();
 
+	if(img->thencheck)
+		return "cannot create case in if-then clause";
+
 	if(String::equal(scr->name, "_init_"))
 		return "this command cannot be used to initialize";
 
@@ -440,6 +477,9 @@ const char *script::checks::chkElif(script *img, script::header *scr, script::li
 {
 	script::method_t method = img->looping();
 
+	if(img->thencheck)
+		return "cannot have elif in if-then clause";
+
 	if(method == (method_t)&script::methods::scrElse) {
 		return "cannot have more if conditions after else";
 	}
@@ -455,6 +495,9 @@ const char *script::checks::chkElse(script *img, script::header *scr, script::li
 {
 	script::method_t method = img->looping();
 
+	if(img->thencheck)
+		return "cannot have else in if-then clause";
+
 	if(method == (method_t)&script::methods::scrElse) {
 		return "cannot have multiple else statements";
 	}
@@ -473,12 +516,13 @@ const char *script::checks::chkElse(script *img, script::header *scr, script::li
 	return NULL;
 }
 
-
-
 const char *script::checks::chkOtherwise(script *img, script::header *scr, script::line_t *line)
 {
 	script::method_t method = img->looping();
 
+	if(img->thencheck)
+		return "cannot have otherwise if-then clause";
+
 	if(String::equal(scr->name, "_init_"))
 		return "this command cannot be used to initialize";
 
@@ -501,6 +545,9 @@ const char *script::checks::chkUntil(script *img, script::header *scr, script::l
 {
 	script::method_t method = img->pull();
 
+	if(img->thencheck)
+		return "cannot have until in if-then clause";
+
 	if(String::equal(scr->name, "_init_"))
 		return "this command cannot be used to initialize";
 
@@ -575,6 +622,9 @@ const char *script::checks::chkDefine(script *img, script::header *scr, script::
 	const char *cp;
 	char idbuf[8];
 
+	if(img->thencheck)
+		return "cannot define in if-then clause";
+
 	if(!line->argc)
 		return NULL;
 
@@ -623,7 +673,7 @@ const char *script::checks::chkPack(script *img, script::header *scr, script::li
 	case '=':
 		return "cannot assign members before symbol name";
 	case '$':
-		return "cannot assign fielded symbols";
+		return "cannot assign to format";
 	case '@':
 		return "cannot assign to label";
 	case '^':
@@ -662,7 +712,7 @@ const char *script::checks::chkPush(script *img, script::header *scr, script::li
 	case '=':
 		return "no keywords are used in this command";
 	case '$':
-		return "cannot assign fielded symbols";
+		return "cannot assign to format";
 	case '@':
 		return "cannot assign to label";
 	case '^':
@@ -717,7 +767,7 @@ const char *script::checks::chkSet(script *img, script::header *scr, script::lin
 	case '=':
 		return "no keywords are used in this command";
 	case '$':
-		return "cannot assign fielded symbols";
+		return "cannot assign to format";
 	case '@':
 		return "cannot assign to label";
 	case '^':
@@ -733,7 +783,7 @@ const char *script::checks::chkSet(script *img, script::header *scr, script::lin
 		if(*cp == '=')
 			return "no keywords are used in this command";
 		if(!isValue(cp))
-			return "cannot assign from label or expression";
+			return "cannot assign to format";
 	}
 	return NULL;
 }
@@ -832,13 +882,10 @@ const char *script::checks::chkGosub(script *img, script::header *scr, script::l
 	if(line->argc < 1)
 		return "gosub requires at least one label";
 
-	if(line->argv[0][0] != '@')
-		return "only allows script section labels";
-
 	while(index < line->argc) {
 		cp = line->argv[index++];
-		if(*cp != '@')
-			return "goto only allows script sections";
+		if(*cp != '@' && !isalnum(*cp))
+			return "gosub only allows script sections and named methods";
 	}
 
 	return NULL;
@@ -867,7 +914,7 @@ const char *script::checks::chkConditional(script *img, script::header *scr, scr
 
 			cp = line->argv[index++];
 
-			if(*cp != '?' && !String::equal(cp, "eq") && !String::equal(cp, "ne") && !String::equal(cp, "lt") && !String::equal(cp, "gt") && !String::equal(cp, "le") && !String::equal(cp, "ge"))
+			if(*cp != '?' && !String::equal(cp, "eq") && !String::equal(cp, "ne") && !String::equal(cp, "lt") && !String::equal(cp, "gt") && !String::equal(cp, "le") && !String::equal(cp, "ge") && !String::equal(cp, "is") && !String::equal(cp, "isnot") && !String::equal(cp, "in") && !String::equal(cp, "notin"))
 				return "invalid operator in expression";
 
 			if(index >= line->argc)
@@ -915,7 +962,7 @@ valid:
 		cp = getArgument(line, &index);
 		if(!cp)
 			break;
-		if(!String::equal(cp, "*") && !String::equal(cp, "/") && !String::equal(cp, "+") && !String::equal(cp, "-"))
+		if(!String::equal(cp, "*") && !String::equal(cp, "/") && !String::equal(cp, "+") && !String::equal(cp, "-") && !String::equal(cp, "#"))
 			return "invalid expression used";
 		if(index == line->argc)
 			return "incomplete expression";
@@ -975,7 +1022,7 @@ const char *script::checks::chkExpr(script *img, script::header *scr, script::li
 		cp = getArgument(line, &index);
 		if(!cp)
 			break;
-		if(!String::equal(cp, "*") && !String::equal(cp, "/") && !String::equal(cp, "+") && !String::equal(cp, "-"))
+		if(!String::equal(cp, "*") && !String::equal(cp, "/") && !String::equal(cp, "+") && !String::equal(cp, "-") && !String::equal(cp, "#"))
 			return "invalid expression used";
 		if(index == line->argc)
 			return "incomplete expression";
