@@ -57,6 +57,9 @@ static bool iskeyword(const char *str)
 	if(!str || !*str)
 		return false;
 
+	if(*str == '_')
+		++str;
+
 	while(*str) {
 		if(!isalpha(*str) && *str != '.')
 			return false;
@@ -73,18 +76,6 @@ static bool isend(const char *str)
 	if(!strncmp("%%", str, 2))
 		return true;
 
-	if(!strncmp("#\n", str, 2))
-		return true;
-
-	if(!strcmp(str, "#"))
-		return true;
-
-	if(!strncmp("# ", str, 2))
-		return true;
-
-	if(!strncmp("#\t", str, 2))
-		return true;
-
 	if(!strncmp("//", str, 2))
 		return true;
 
@@ -123,6 +114,13 @@ static bool preparse(char **tokens)
 		return false;
 #endif
 	case '>':
+		if(String::equal(str, ">>%", 3)) {
+			--*tokens;
+			--str;
+			str[0] = '$';
+			str[3] = ':';
+			return true;
+		}
 		if(isspace(str[1]) || (str[1] == '=' && isspace(str[2]))) {
 			--*tokens;
 			**tokens = '?';
@@ -144,6 +142,13 @@ static bool preparse(char **tokens)
 		}
 		return false;
 	case '<':
+		if(String::equal(str, "<<%", 3)) {
+			--*tokens;
+			--str;
+			str[0] = '$';
+			str[3] = ':';
+			return true;
+		}
 		if(isspace(str[1]) || (str[1] == '=' && isspace(str[2])) || (str[1] == '>' && isspace(str[2]))) {
 			--*tokens;
 			**tokens = '?';
@@ -212,6 +217,13 @@ static bool preparse(char **tokens)
 	case '_':
 		return false;
 	case '+':
+		if(String::equal(str, "++%", 3)) {
+			--*tokens;
+			--str;
+			str[0] = '$';
+			str[3] = ':';
+			return true;
+		}
 		if(isdigit(str[1]) || str[1] == '.') {
 			++*tokens;
 			return true;
@@ -226,6 +238,13 @@ static bool preparse(char **tokens)
 			break;
 		return false;
 	case '-':
+		if(String::equal(str, "--%", 3)) {
+			--*tokens;
+			--str;
+			str[0] = '$';
+			str[3] = ':';
+			return true;
+		}
 		if(isalnum(str[1]) || str[1] == '.')
 			return true;
 		if(isspace(str[1]))
@@ -269,6 +288,7 @@ CountedObject(), memalloc()
 {
 	errors = 0;
 	loop = 0;
+	lines = 0;
 	first = NULL;
 	global = NULL;
 	headers = NULL;
@@ -359,6 +379,7 @@ void script::init(void)
 		{"_ifthen", (method_t)&methods::scrWhen, (check_t)&checks::chkWhen},
 		{"_define", (method_t)&methods::scrDefine, (check_t)&checks::chkDefine},
 		{"_invoke", (method_t)&methods::scrInvoke, (check_t)&checks::chkInvoke},
+		{"_ref", (method_t)&methods::scrRef, (check_t)&checks::chkRef},
 		{NULL}
 	};
 
@@ -384,6 +405,28 @@ script::header *script::find(script *img, const char *id)
 	return *hp;
 }
 
+script *script::merge(const char *fn, script *root)
+{
+	script *img = compile(fn, root);
+	if(!root || !img)
+		return img;
+
+	for(unsigned index = 0; index < script::indexing; ++index) {
+		header *prior = NULL;
+		linked_pointer<header> hp = img->scripts[index];
+		while(is(hp)) {
+			prior = *hp;
+			hp.next();
+		}
+		if(prior)
+			prior->link(static_cast<header *>(root->scripts[index]));
+		else
+			img->scripts[index] = root->scripts[index];
+	}
+
+	return img;
+}	
+	
 script *script::compile(const char *fn, script *cfg)
 {
 //	linked_pointer<script::strict> sp;
@@ -402,7 +445,8 @@ script *script::compile(const char *fn, script *cfg)
 	line_t *line, *last;
 	bool section = false;
 	unsigned argc;
-	const char *err;
+	const char *err, *op;
+	char *assigned;
 	unsigned path;
 	bool indented, formed;
 	script::header *invoke;
@@ -412,6 +456,8 @@ script *script::compile(const char *fn, script *cfg)
 	unsigned pos;
 	const char *cp;
 	bool requires = false;
+	char *ep;
+	unsigned len;
 
 	if(!fp)
 		return NULL;
@@ -443,6 +489,7 @@ initial:
 	scr->scoped = NULL;
 	keyword_t *keyword;
 	img->loop = 0;
+	char *temp;
 
 	path = NamedObject::keyindex(name, script::indexing);
 	scr->enlist(&img->scripts[path]);
@@ -471,6 +518,9 @@ initial:
 			goto parse;
 		}
 
+		img->lines = ++lnum;
+		img->thencheck = false;
+
 		indented = false;
 		tokens = buffer.c_mem();
 		if(isspace(*tokens)) {
@@ -484,8 +534,7 @@ initial:
 		}
 
 		String::trim(buffer, " \t\r\n");
-		++lnum;
-
+		
 		// if empty line or comment, continue...
 		if(isend(*buffer))
 			continue;
@@ -532,16 +581,19 @@ initial:
 		if(requires)
 			continue;
 
-		if(*token == '^') {
+		if(*token == '^' || *token == '-') {
 			current = (event *)img->alloc(sizeof(script::event));	
 			if(!prior)
 				prior = current;
 
 			last = NULL;
 			memset(current, 0, sizeof(event));
+			if(*token == '^')
+				current->enlist(&scr->events);
+			else
+				current->enlist(&scr->methods);
 			current->name = (char *)img->dup(++token);
 			current->first = NULL;
-			current->enlist(&scr->events);
 			continue;
 		}
 
@@ -593,6 +645,34 @@ initial:
 		}
 
 parse:
+		assigned = NULL;
+		op = NULL;
+		if(*token == '%') {
+			assigned = img->dup(token);
+			op = String::token(NULL, &tokens, " \t", "{}\'\'\"\"");
+			if(op && (String::equal(op, ":=") || String::equal(op, "+=") || String::equal(op, "-=") || String::equal(op, "*=") || String::equal(op, "/=") || String::equal(op, "%=") || String::equal(op, "#=")))
+				token = "expr";
+			else if(op && (String::equal(op, "=") || String::equal(op, "==") || String::equal(op, "$="))) {
+				op = NULL;
+				token = "set";
+			}
+			else if(op && (String::equal(op, ".") || String::equal(op, ".="))) {
+				op = NULL;
+				token = "add";
+			}
+			else if(op && (String::equal(op, ",") || String::equal(op, ",="))) {
+				op = NULL;
+				token = "pack";
+			} 
+			else {
+				img->errlog(lnum, "invalid assignment");
+				continue;
+			}
+		}
+		else if(*token == '$') {
+			assigned = img->dup(token);
+			token = "_ref";
+		}
 		invoke = NULL;
 		if(!iskeyword(token)) {
 			img->errlog(lnum, "invalid keyword");
@@ -620,6 +700,9 @@ parse:
 			}
 		}
 
+		// protects in backparse
+		token = img->dup(token);
+
 		line = (line_t *)img->alloc(sizeof(line_t));
 		memset(line, 0, sizeof(line_t));
 		line->next = NULL;
@@ -628,14 +711,20 @@ parse:
 		if(invoke)
 			line->sub = invoke;
 		else
-			line->cmd = img->dup(token);
+			line->cmd = token;
 		line->loop = img->loop;
 		line->method = keyword->method;
 
 		formed = true;
-
 		argc = 0;
 		arg = NULL;
+
+		if(assigned)
+			argv[argc++] = assigned;
+		
+		if(op)
+			argv[argc++] = img->dup(op);
+
 		while(argc < 249 && formed) {
 			if(!arg) {
 				formed = preparse(&tokens);
@@ -655,20 +744,71 @@ parse:
 					keyword = find("_ifthen");
 					line->method = keyword->method;
 					token = String::token(NULL, &tokens, " \t", "{}\'\'\"\"");
-					if(!token || !keyword) 
+					if(!token || !keyword || img->thencheck) 
 						formed = false;
 					else
-						when = true;
+						img->thencheck = when = true;
 					break;
 				}
 			}
 
+			ep = strchr(arg, '<');
+			if(*arg == '%' && ep) {
+				len = strlen(arg) + 10;
+				*(ep++) = 0;
+				temp = (char *)img->alloc(len);
+				String::set(temp, len, "$map/");
+				String::add(temp, len, ep);
+				ep = strchr(temp, '>');
+				if(ep)
+					*ep = ':';
+				String::add(temp, len, ++arg);
+				argv[argc++] = temp;
+				arg = NULL;
+				continue;
+			}
+
+			ep = strchr(arg, '[');
+			if(*arg == '%' && ep) {
+				len = strlen(arg) + 10;
+				*(ep++) = 0;
+				temp = (char *)img->alloc(len);
+				if(atoi(ep))
+					String::set(temp, len, "$offset/");
+				else
+					String::set(temp, len, "$find/");
+				String::add(temp, len, ep);
+				ep = strchr(temp, ']');
+				if(ep)
+					*ep = ':';
+				String::add(temp, len, ++arg);
+				argv[argc++] = temp;
+				arg = NULL;
+				continue;
+			}
+
+			ep = strchr(arg, '(');
+			if(*arg == '%' && ep) {
+				len = strlen(arg) + 10;
+				*(ep++) = 0;
+				temp = (char *)img->alloc(len);
+				String::set(temp, len, "$index/");
+				String::add(temp, len, ep);
+				ep = strchr(temp, ')');
+				if(ep)
+					*ep = ':';
+				String::add(temp, len, ++arg);
+				argv[argc++] = temp;
+				arg = NULL;
+				continue;
+			}
+
 			argv[argc++] = img->dup(arg);
 			arg = NULL;
 		}
 
 		if(!formed) {
-			img->errlog(lnum, "malformed argument");
+			img->errlog(lnum, "malformed statement or argument");
 			continue;
 		}
 
@@ -922,7 +1062,9 @@ bool script::strict::find(script* image, script::header *scr, const char *id)
 	assert(scr != NULL);
 	assert(image != NULL);
 
+	char buf[64];
 	const char *cp;
+	char *ep;
 	linked_pointer<script::strict> sp;
 
 	if(!image->global)
@@ -931,6 +1073,17 @@ bool script::strict::find(script* image, script::header *scr, const char *id)
 	if(*id != '%' && *id != '$')
 		return true;
 
+	if(String::equal(id, "$map/", 5)) {
+		buf[0] = '%';
+		String::set(buf + 1, sizeof(buf) - 1, id + 5);
+		ep = strchr(buf, ':');
+		if(ep)
+			*ep = 0;
+
+		if(!find(image, scr, buf))
+			return false;
+	}
+
 	if(*id == '$') {
 		cp = strchr(id, ':');
 		if(cp)
