@@ -95,16 +95,6 @@ char *script::interp::getTemp(void)
 	return ptr;
 }
 
-const char *script::interp::listOffset(const char *list, unsigned index)
-{
-	return script::get(list, index);
-}
-		
-unsigned script::interp::listCount(const char *data)
-{
-	return script::count(data);
-}
-
 const char *script::interp::getFormat(script::symbol *sym, const char *id, char *tp)
 {
 #ifdef	__MINGW32__
@@ -120,6 +110,7 @@ const char *script::interp::getFormat(script::symbol *sym, const char *id, char
 	unsigned len;
 	char quote = 0;
 	script::symbol *index = NULL;
+	unsigned paren = 0;
 
 	cp = sym->data;
 	while(isspace(*cp))
@@ -150,7 +141,7 @@ const char *script::interp::getFormat(script::symbol *sym, const char *id, char
 				tp[--dcount] = 0;
 		}
 	}
-	else if(String::equal(id, "pull:", 5)) {
+	else if(String::equal(id, "pull:", 5) || String::equal(id, "<<:", 3)) {
 		String::set(tp, script::sizing + 1, sym->data);
 		if(script::count(tp) > 1) {
 			dcount = script::offset(tp, 1);
@@ -160,7 +151,7 @@ const char *script::interp::getFormat(script::symbol *sym, const char *id, char
 			}
 		}
 	}
-	else if(String::equal(id, "pop:", 4)) {
+	else if(String::equal(id, "pop:", 4) || String::equal(id, ">>:", 3)) {
 		dcount = script::count(sym->data);
 		if(dcount) {
 			String::set(tp, script::sizing + 1, script::get(sym->data, --dcount));
@@ -225,7 +216,7 @@ const char *script::interp::getFormat(script::symbol *sym, const char *id, char
 		String::lower(sym->data);
 		return sym->data;
 	}	
-	else if(String::equal(id, "inc:", 4) && sym->size) {
+	else if((String::equal(id, "inc:", 4) || String::equal(id, "++:", 3)) && sym->size) {
 		val = atol(cp);
 		snprintf(sym->data, sym->size + 1, NUM_FORMAT, ++val);
 		return sym->data;
@@ -237,17 +228,8 @@ const char *script::interp::getFormat(script::symbol *sym, const char *id, char
 		pos = atoi(id);
 		if(pos)
 			--pos;
-		else {
-			snprintf(tp, script::sizing + 1, "%s", id);
-			ep = strchr(tp, ':');
-			*ep = 0;
-			index = find(tp);
-			if(!index)
-				return "";
-			pos = atoi(index->data);
-			if(pos)
-				--pos;
-		}
+		else 
+			return "";
 		if(pos < strlen(sym->data))
 			cp = sym->data + pos;
 		else
@@ -259,21 +241,13 @@ const char *script::interp::getFormat(script::symbol *sym, const char *id, char
 		pos = atoi(id);
 		if(pos)
 			--pos;
-		else {
-			snprintf(tp, script::sizing + 1, "%s", id);
-			ep = strchr(tp, ':');
-			*ep = 0;
-			index = find(tp);
-			if(!index)
-				return "";
-			pos = atoi(index->data);
-			if(pos)
-				--pos;
-		}
+		else 
+			return "";
+offset:
 		cp = script::get(sym->data, pos);
 		script::copy(cp, tp, script::sizing);
 	}
-	else if(String::equal(id, "dec:", 4) && sym->size) {
+	else if((String::equal(id, "dec:", 4) || String::equal(id, "--:", 3)) && sym->size) {
 		val = atoi(cp);
 		snprintf(sym->data, sym->size + 1, NUM_FORMAT, --val);
 		return sym->data;
@@ -303,59 +277,58 @@ const char *script::interp::getFormat(script::symbol *sym, const char *id, char
 		}
 		tp[pos] = 0; 		
 	}
+	else if(String::equal(id, "map/", 4)) {
+		idbuf[0] = '%';
+		String::set(idbuf, sizeof(idbuf), id + 4);
+		ep = strchr(idbuf, ':');
+		if(ep)
+			*ep = 0;
+		index = find(idbuf);
+		if(!index || !index->data[0])
+			return "";
+		pos = atoi(sym->data);
+		if(pos)
+			goto offset;
+		snprintf(idbuf, sizeof(idbuf), ",%s", sym->data);
+		paren = 0;
+		goto search;
+	}
 	else if(String::equal(id, "find/", 5)) {
 		id += 5;
-		dcount = atoi(id);
-		if(dcount) {
-			cp = sym->data;
-			while(dcount > 1 && *cp) {
-				if(*cp == '\"') {
-					quote = *cp;
-					++cp;
-				}
-				else
-					quote = ',';
-				while(*cp && *cp != quote) {
-					if(*cp == '=' && quote == ',')
-						break;
-					++cp;
-				}
-				if(*cp == '=' && quote == ',') {
-					++cp;
-					continue;
-				}
-				if(quote != ',') {
-					++cp;
-					continue;
-				}
-				--dcount;
-				++cp;
-			}
-		}
-		else {
-			dcount = 1;
-			idbuf[0] = ',';
-			while(dcount < 30 && *id && *id != ':')
-				idbuf[dcount++] = *(id++);
-			idbuf[dcount++] = '=';
-			idbuf[dcount] = 0;
-			cp = sym->data;
-			len = strlen(idbuf);
-			if(!strncmp(cp, idbuf + 1, len - 1))
-				cp = sym->data + len - 1;
-			else if(NULL != (cp = strstr(sym->data, idbuf)))
-				cp = cp + len;
-			else
-				cp = "";
-		}
+		paren = 0;
+		dcount = 1;
+		idbuf[0] = ',';
+		while(dcount < 30 && *id && *id != ':')
+			idbuf[dcount++] = *(id++);
+		idbuf[dcount++] = '=';
+		idbuf[dcount] = 0;
+search:
+		cp = sym->data;
+		len = strlen(idbuf);
+		if(!strncmp(cp, idbuf + 1, len - 1))
+			cp = sym->data + len - 1;
+		else if(NULL != (cp = strstr(sym->data, idbuf)))
+			cp = cp + len;
+		else
+			cp = "";
 		dcount = 0;
 		if(*cp == '\"') {
 			quote = *cp;
 			++cp;
 		}
+		else if(*cp == '(') {
+			paren = 1;
+			++cp;
+		}
 		else
 			quote=',';
 		while(dcount < script::sizing && *cp && *cp != quote) {
+			if(*cp == '(' && paren)
+				++paren;
+			if(*cp == ')' && paren) {
+				if(paren-- == 1)
+					break;
+			}
 			if(*cp == '=' && quote == ',' && cp[1] == '\"') {
 				tp[dcount++] = *(cp++);
 				quote = *(cp++);
@@ -864,6 +837,18 @@ bool script::interp::isInherited(const char *name)
 	return true;
 }
 
+script::event *script::interp::scriptMethod(const char *name)
+{
+	linked_pointer<script::event> mp = stack[frame].scr->methods;
+
+	while(is(mp)) {
+		if(match(mp->name, name))
+			return *mp;
+		mp.next();
+	}
+	return NULL;
+}
+
 bool script::interp::scriptEvent(const char *name)
 {
 	linked_pointer<script::event> ep;
@@ -1185,9 +1170,9 @@ bool script::interp::getExpression(unsigned index)
 	const char *v1 = "", *v2 = "", *op;
 	const char *d1, *d2;
 	unsigned len;
-	char quote = 0;
 	char dec1[9], dec2[9];
 	long dv1, dv2;
+	unsigned pos = 0, count;
 
 #ifdef	__MINGW32__
 	long long dv;
@@ -1329,52 +1314,38 @@ bool script::interp::getExpression(unsigned index)
 		return (atol(v1) > atol(v2) || (atol(v1) == atol(v2) && dv1 >= dv2));
 	}
 
-	if(String::equal(op, "?")) {
-		op = v2;
+	if(String::equal(op, "?") || String::equal(op, "in")) {
+		count = script::count(v2);
+		if(!count)
+			return false;
+
 		len = strlen(v1);
-		while(op && *op) {
-			if(*op == '\"')
-				quote = *(op++);
-			else
-				quote = 0;
-			if(!strncmp(op, v1, len)) {
-				if(op[len] == quote || op[len] == '=' || op[len] == ',')
-					return true;
-			}
-			if(quote)
-				op = strchr(op, quote);
-			op = strchr(op, ',');
-			if(op)
-				++op;
+		while(pos < count) {
+			op = script::get(v2, pos++);
+			if(op && String::equal(v1, op, len) && (op[len] == ',' || op[len] == 0 || op[len] == '='))
+				return true;
 		}
 		return false;
 	}
 
-	if(String::equal(op, "!?")) {
-		op = v2;
+	if(String::equal(op, "!?") || String::equal(op, "notin")) {
+		count = script::count(v2);
+		if(!count)
+			return true;
+
 		len = strlen(v1);
-		while(op && *op) {
-			if(*op == '\"')
-				quote = *(op++);
-			else
-				quote = 0;
-			if(!strncmp(op, v1, len)) {
-				if(op[len] == quote || op[len] == '=' || op[len] == ',')
-					return false;
-			}
-			if(quote)
-				op = strchr(op, quote);
-			op = strchr(op, ',');
-			if(op)
-				++op;
+		while(pos < count) {
+			op = script::get(v2, pos++);
+			if(op && String::equal(v1, op, len) && (op[len] == ',' || op[len] == 0 || op[len] == '='))
+				return false;
 		}
 		return true;
 	}
 
-	if(String::equal(op, "!$"))
+	if(String::equal(op, "!$") || String::equal(op, "isnot"))
 		return !match(v2, v1);
 
-	if(String::equal(op, "$"))
+	if(String::equal(op, "$") || String::equal(op, "is"))
 		return match(v2, v1);
 
 #ifdef	HAVE_REGEX_H
